#include <stdio.h>
#include <string.h>
#include "comando.h"
#include "plano.h"

// Função para movimentar retangulo numa direcção
void moverRetangulo(Rectangle** retangulos, int numRetangulos, int x, int y, int p, int direcao) {
    // Percorre todos os retângulos
    for (int j = 0; j < numRetangulos; j++) {
        // Verifica se o ponto (x, y) está dentro do retângulo atual
        if (x >= retangulos[j]->x && x < retangulos[j]->x + retangulos[j]->l &&
            y >= retangulos[j]->y && y < retangulos[j]->y + retangulos[j]->h) {
            // Move o retângulo na direção especificada
            if (direcao == 1 && retangulos[j]->x + retangulos[j]->l + p <= LARGURA_MUNDO) {
                retangulos[j]->x += p; // Move para a direita
            } else if (direcao == -1 && retangulos[j]->x - p >= 0) {
                retangulos[j]->x -= p; // Move para a esquerda
            } else {
                printf("Erro: Movimento fora dos limites do plano.\n");
            }
            break;
        }
    }
}

// Verifica se um retângulo com as dimensões especificadas está dentro dos limites do plano
int estaDentroDosLimites(int x, int y, int l, int h) {
    return (x >= 0 && x + l <= LARGURA_MUNDO && y >= 0 && y + h <= ALTURA_MUNDO);
}

// Executa um comando sobre os retângulos
void runComando(char* comando, Rectangle** retangulos, int* numRetangulos) {
    char* tipoComando = strtok(comando, " ");
    if (tipoComando == NULL) {
        printf("Comando vazio!\n");
        return;
    }

    // Verifica o tipo de comando
    if (strcmp(tipoComando, "create") == 0) { // Comando para criar um retângulo
        char* parametros = strtok(NULL, "");
        int x, y, l, h;

        if (sscanf(parametros, "%d,%d+%d,%d", &x, &y, &l, &h) == 4) {
            // Verifica se o retângulo está dentro dos limites do plano
            if (estaDentroDosLimites(x, y, l, h)) {
                int sobrepoe = 0;
                // Verifica se o retângulo se sobrepõe a outros retângulos já existentes
                for (int i = 0; i < *numRetangulos; i++) {
                    Rectangle* rect = retangulos[i];
                    if (!(x + l <= rect->x || x >= rect->x + rect->l || y + h <= rect->y || y >= rect->y + rect->h)) {
                        sobrepoe = 1;
                        break;
                    }
                }
                if (!sobrepoe) {
                    retangulos[(*numRetangulos)++] = createRectangle(x, y, l, h);
                } else {
                    printf("Erro: Já existe um retângulo nessas coordenadas.\n");
                }
            } else {
                printf("Erro: Retângulo fora dos limites do plano. Verifique as coordenadas e dimensões.\n");
            }
        } else {
            printf("Erro: Formato de comando 'create' inválido. Use: create x,y+l,h\n");
        }
    } else if (strcmp(tipoComando, "moveright") == 0 || strcmp(tipoComando, "moveleft") == 0) {
        // Comando para mover um retângulo para a direita ou esquerda
        char* parametros = strtok(NULL, "");
        int x, y, p;
        int direcao = (strcmp(tipoComando, "moveright") == 0) ? 1 : -1;

        if (sscanf(parametros, "%d,%d+%d", &x, &y, &p) == 3) {
            moverRetangulo(retangulos, *numRetangulos, x, y, p, direcao);
        } else {
            printf("Erro: Formato de comando inválido. Use: moveright x,y+p ou moveleft x,y+p\n");
        }
    } else {
        printf("Comando desconhecido: %s\n", tipoComando);
    }

    // Aplica a gravidade aos retângulos se houver retângulos no plano
    if (*numRetangulos > 0) {
        for (int i = 0; i < *numRetangulos; i++) {
            gravity(retangulos[i], retangulos, *numRetangulos);
        }
    }
}