#include <stdio.h>
#include <string.h>
#include "comando.h"
#include "plano.h"

// Função para verificar a sobreposição entre dois retângulos
int verificaSobreposicao(Rectangle* rect1, Rectangle* rect2) {
    if (rect1->x == rect2->x && rect1->l == rect2->l) {
        if (rect1->y <= rect2->y + rect2->h && rect2->y <= rect1->y + rect1->h) {
            return 1; // Sobreposição encontrada
        }
    }
    return 0; // Sem sobreposição
}

// Função para alertar sobre a sobreposição entre os retângulos
void alertaSobreposicao(Rectangle** retangulos, int numRetangulos) {
    for (int i = 0; i < numRetangulos - 1; i++) {
        for (int j = i + 1; j < numRetangulos; j++) {
            if (verificaSobreposicao(retangulos[i], retangulos[j])) {
                printf("Alerta: Existe sobreposição entre retângulos. Use o comando merge\n");
            }
        }
    }
}

// Função para buscar um retângulo com base em suas coordenadas
Rectangle* buscarRetanguloPorCoordenadas(Rectangle** retangulos, int numRetangulos, int x, int y) {
    for (int i = 0; i < numRetangulos; i++) {
        Rectangle* rect = retangulos[i];
        if (x >= rect->x && x <= rect->x + rect->l &&
            y >= rect->y && y <= rect->y + rect->h) {
            return rect; // Retângulo encontrado
        }
    }
    return NULL; // Retângulo não encontrado
}
// Função para movimentar retangulo numa direcção
void moverRetangulo(Rectangle** retangulos, int numRetangulos, int x, int y, int p, int direcao) {
    // Percorre todos os retângulos
    for (int j = 0; j < numRetangulos; j++) {
        // Verifica se o ponto (x, y) está dentro do retângulo atual
        if (x >= retangulos[j]->x && x < retangulos[j]->x + retangulos[j]->l &&
            y >= retangulos[j]->y && y < retangulos[j]->y + retangulos[j]->h) {
            // Move o retângulo na direção especificada
            if (direcao == 1 && retangulos[j]->x + retangulos[j]->l + p <= LARGURA_MUNDO) {
                retangulos[j]->x += p; // Move para a direita
                printf("Retângulo movido para a direita com sucesso.\n");
            } else if (direcao == -1 && retangulos[j]->x - p >= 0) {
                retangulos[j]->x -= p; // Move para a esquerda
                printf("Retângulo movido para a esquerda com sucesso.\n");
            } else {
                printf("Erro: Movimento fora dos limites do plano.\n");
            }
            break;
        }
    }
}

// Verifica se um retângulo com as dimensões especificadas está dentro dos limites do plano
int estaDentroDosLimites(int x, int y, int l, int h) {
    return (x >= 0 && x + l <= LARGURA_MUNDO && y >= 0 && y + h <= ALTURA_MUNDO);
}

// Executa um comando sobre os retângulos
void runComando(char* comando, Rectangle** retangulos, int* numRetangulos) {
    char* tipoComando = strtok(comando, " ");
    if (tipoComando == NULL) {
        printf("Comando vazio!\n");
        return;
    }

    // Verifica o tipo de comando
    if (strcmp(tipoComando, "create") == 0) { 
        // Comando para criar um retângulo
        char* parametros = strtok(NULL, "");
        int x, y, l, h;

        if (sscanf(parametros, "%d,%d+%d,%d", &x, &y, &l, &h) == 4) {
            if (estaDentroDosLimites(x, y, l, h)) {
                int sobrepoe = 0;
                // Verifica se o retângulo se sobrepõe a outros retângulos já existentes
                for (int i = 0; i < *numRetangulos; i++) {
                    Rectangle* rect = retangulos[i];
                    if (!(x + l <= rect->x || x >= rect->x + rect->l || y + h <= rect->y || y >= rect->y + rect->h)) {
                        sobrepoe = 1;
                        break;
                    }
                }
                if (!sobrepoe) {
                    retangulos[(*numRetangulos)++] = createRectangle(x, y, l, h);
                    printf("Retângulo criado com sucesso.\n");
                } else {
                    printf("Erro: Já existe um retângulo nessas coordenadas.\n");
                }
            } else {
                printf("Erro: Retângulo fora dos limites do plano. Verifique as coordenadas e dimensões.\n");
            }
        } else {
            printf("Erro: Formato de comando 'create' inválido. Use: create x,y+l,h\n");
        }
    } else if (strcmp(tipoComando, "moveright") == 0 || strcmp(tipoComando, "moveleft") == 0) {
        // Comando para mover um retângulo para a direita ou esquerda
        char* parametros = strtok(NULL, "");
        int x, y, p;
        int direcao = (strcmp(tipoComando, "moveright") == 0) ? 1 : -1;

        if (sscanf(parametros, "%d,%d+%d", &x, &y, &p) == 3) {
            moverRetangulo(retangulos, *numRetangulos, x, y, p, direcao);
        } else {
            printf("Erro: Formato de comando inválido. Use: moveright x,y+p ou moveleft x,y+p\n");
        }
    } else if (strcmp(tipoComando, "merge") == 0 || strcmp(tipoComando, "mergePossivel") == 0) {
        // Comando para unir ou verificar se da para unir dois retângulos
        char* parametros = strtok(NULL, "");
        int x1, y1, x2, y2;

        if (sscanf(parametros, "%d,%d+%d,%d", &x1, &y1, &x2, &y2) == 4) {
            Rectangle* rect1 = buscarRetanguloPorCoordenadas(retangulos, *numRetangulos, x1, y1);
            Rectangle* rect2 = buscarRetanguloPorCoordenadas(retangulos, *numRetangulos, x2, y2);

            if (rect1 == NULL || rect2 == NULL) {
                printf("Erro: Pelo menos um dos retângulos não foi encontrado.\n");
                return;
            }

            if (verificaSobreposicao(rect1, rect2)) {
                if (strcmp(tipoComando, "merge") == 0) {
                    int minY = (rect1->y < rect2->y) ? rect1->y : rect2->y;
                    int maxY = (rect1->y + rect1->h > rect2->y + rect2->h) ? rect1->y + rect1->h : rect2->y + rect2->h;
                    rect1->y = minY;
                    rect1->h = maxY - minY;

                    for (int i = 0; i < *numRetangulos; i++) {
                        if (retangulos[i] == rect2) {
                            freeRectangle(rect2);
                            for (int j = i; j < *numRetangulos - 1; j++) {
                                retangulos[j] = retangulos[j + 1];
                            }
                            (*numRetangulos)--;
                            break;
                        }
                    }

                    printf("Retângulos unidos com sucesso.\n");
                } else {
                    printf("Sim, esses retângulos podem ser unidos.\n");
                }
            } else {
                if (strcmp(tipoComando, "merge") == 0) {
                    printf("Erro: Os retângulos não podem ser unidos.\n");
                } else {
                    printf("Não, esses retângulos não podem ser unidos.\n");
                }
            }
        } else {
            printf("Erro: Formato de comando inválido. Use: merge x1,y1+x2,y2 ou mergePossivel x1,y1+x2,y2\n");
        }
    } else {
        printf("Comando desconhecido: %s\n", tipoComando);
    }

    // Aplica a gravidade aos retângulos se houver retângulos no plano
    if (*numRetangulos > 0) {
        for (int i = 0; i < *numRetangulos; i++) {
            gravity(retangulos[i], retangulos, *numRetangulos);
        }
    }
}