#include <stdlib.h>
#include "rectangle.h"

// Função para criar um retângulo com as coordenadas e dimensões especificadas
Rectangle* createRectangle(int x, int y, int l, int h) {
    Rectangle* rect = (Rectangle*)malloc(sizeof(Rectangle)); // Aloca memória para o retângulo
    if (rect != NULL) { // Verifica se a alocação foi bem-sucedida
        rect->x = x; // Define a coordenada x do retângulo
        rect->y = y; // Define a coordenada y do retângulo
        rect->l = l; // Define a largura do retângulo
        rect->h = h; // Define a altura do retângulo
    }
    return rect; // Retorna o ponteiro para o retângulo criado
}

// Função para liberar a memória alocada para um retângulo
void freeRectangle(Rectangle* rect) {
    free(rect); // Libera a memória do retângulo
}

// Função para aplicar a gravidade a um retângulo
void gravity(Rectangle* rect, Rectangle** retangulos, int numRetangulos) {
    int maxDescer = rect->y; // Inicializa a variável para o máximo de pixels que o retângulo pode descer

    // Percorre todas as posições possíveis de descida do retângulo
    for (int j = 1; j <= maxDescer; j++) {
        int colisao = 0; // Variável para indicar se houve colisão durante a descida
        // Verifica se há colisão com algum outro retângulo na posição atual de descida
        for (int i = 0; i < numRetangulos; i++) {
            if (rect != retangulos[i] &&
                rect->x < retangulos[i]->x + retangulos[i]->l &&
                rect->x + rect->l > retangulos[i]->x &&
                rect->y - j < retangulos[i]->y + retangulos[i]->h &&
                rect->y - j >= retangulos[i]->y) {
                maxDescer = j - 1; // Atualiza o máximo de pixels que o retângulo pode descer
                colisao = 1; // Indica que houve colisão
                break; // Sai do loop, pois já houve colisão
            }
        }
        if (colisao) {
            break; // Se houve colisão, interrompe a verificação de descida
        }
    }

    rect->y -= maxDescer; // Aplica a descida máxima permitida ao retângulo
}